﻿using System;
using Amplitude.AI.Common.DataStructure;
using Amplitude.AI.Core.Brain.Tasks;
using Amplitude.AI.Core.Decision.ProcessingPasses;
using Amplitude.AI.Core.Framework.Debug;
using Amplitude.AI.Core.Snapshots;
using Amplitude.AI.Core.Snapshots.EntitySnapshots.Empire;
using Amplitude.AI.Core.Snapshots.EntitySnapshots.ShipDesign;

namespace Amplitude.AI.Core.Brain.Generators.ShipDesign
{
    public class SmallInvaderRoleMissing : DefaultRootGenerator<ShipDesignLayoutContextData, CreateShipDesign>
    {
        public SmallInvaderRoleMissing()
        {
            this.idxDone = -1;
        }

        public override string Description
        {
            get
            {
                return "Push a invasion design when the ship design is correct.";
            }
        }

        public override int EvaluationPriority
        {
            get
            {
                return 0;
            }
        }

        public override string LocalizedName
        {
            get
            {
                return "small invader design production";
            }
        }

        protected override void GetContexts(AIPlayer aiPlayer)
        {
            if (aiPlayer.Blackboard.Any<CreateShipDesign>(null))
            {
                return;
            }
            if (this.idxDone == aiPlayer.ControlledEmpireIndex)
            {
                return;
            }
            MajorEmpireSnapshot majorEmpireSnapshot = aiPlayer.ControlledEmpireSnapshot as MajorEmpireSnapshot;
            if (majorEmpireSnapshot == null)
            {
                return;
            }
            for (int i = 0; i < aiPlayer.AnalysisDataHolder.MyEmpireData.AvailableShipDesigns.Count; i++)
            {
                ShipDesignSnapshot shipDesignSnapshot = aiPlayer.AnalysisDataHolder.MyEmpireData.AvailableShipDesigns[i];
                if ((shipDesignSnapshot.ShipRole == ShipPattern.ShipRole.Invader || shipDesignSnapshot.LocalizedName == "S.Invader") && shipDesignSnapshot.CommandPoints == 1f)
                {
                    this.idxDone = aiPlayer.ControlledEmpireIndex;
                    return;
                }
            }
            for (int j = 0; j < majorEmpireSnapshot.ShipLayoutContextData.Count; j++)
            {
                ShipDesignLayoutContextData shipDesignLayoutContextData = majorEmpireSnapshot.ShipLayoutContextData[j];
                bool flag = false;
                for (int k = 0; k < majorEmpireSnapshot.AvailableHulls.Count; k++)
                {
                    if (majorEmpireSnapshot.AvailableHulls[k].Name == shipDesignLayoutContextData.HullName)
                    {
                        flag = true;
                        break;
                    }
                }
                if (flag)
                {
                    bool flag2 = false;
                    for (int l = 0; l < shipDesignLayoutContextData.AvailableModules.Length; l++)
                    {
                        ShipLayoutModuleSnapshot shipLayoutModuleSnapshot = shipDesignLayoutContextData.AvailableModules[l];
                        if (shipLayoutModuleSnapshot.ModuleName.Contains("ModuleSupportCrew"))
                        {
                            flag2 = true;
                        }
                    }
                    if (flag2 && shipDesignLayoutContextData.Role == "ShipRoleSmallSupport")
                    {
                        base.ContinueDecision(aiPlayer, shipDesignLayoutContextData);
                        return;
                    }
                }
            }
        }

        protected override void GenerateDecision(AIPlayer aiPlayer, ShipDesignLayoutContextData layout)
        {
            CreateShipDesign createShipDesign = aiPlayer.Blackboard.FindFirstRunningHull<CreateShipDesign>(layout);
            if (createShipDesign == null)
            {
                string userName = "S.Invader";
                createShipDesign = new CreateShipDesign(aiPlayer.Blackboard.GenerateGUID(), userName, layout.HullName, layout.Role);
                createShipDesign.ModuleBySlot = this.ApplyModules(layout);
                if (createShipDesign.ModuleBySlot == null)
                {
                    return;
                }
                this.idxDone = aiPlayer.ControlledEmpireIndex;
                aiPlayer.Blackboard.Register<CreateShipDesign>(createShipDesign);
            }
            base.ContinueDecision(aiPlayer, base.GetGenerationData(layout, createShipDesign));
        }

        protected override HeuristicFloat ComputeLocalMotivation(AIPlayer aiPlayer, GenerationData<ShipDesignLayoutContextData, CreateShipDesign> generationData)
        {
            return new HeuristicFloat(1f, "");
        }

        private string[] ApplyModules(ShipDesignLayoutContextData layout)
        {
            if (layout.Slots == null || layout.Slots.Length == 0)
            {
                return null;
            }
            string[] array = new string[layout.Slots.Length];
            for (int i = 0; i < layout.Slots.Length; i++)
            {
                array[i] = string.Empty;
            }
            int num = -1;
            for (int j = 0; j < layout.AvailableModules.Length; j++)
            {
                ShipLayoutModuleSnapshot shipLayoutModuleSnapshot = layout.AvailableModules[j];
                if (shipLayoutModuleSnapshot.ModuleName.Contains("ModuleSupportCrew"))
                {
                    num = j;
                    break;
                }
            }
            for (int k = 0; k < array.Length; k++)
            {
                array[k] = string.Empty;
                if (!layout.Slots[k].IsHidden && layout.Slots[k].ArePrerequisitesValid && layout.ModuleBySlot[k].Contains(layout.AvailableModules[num].ModuleName))
                {
                    array[k] = layout.AvailableModules[num].ModuleName;
                    break;
                }
            }
            return array;
        }

        private int idxDone;
    }
}
