﻿using System;
using System.Collections.Generic;
using Amplitude.AI.Common.DataStructure;
using Amplitude.AI.Core.Brain.Analysis.Data;
using Amplitude.AI.Core.Brain.Strategic.Data;
using Amplitude.AI.Core.DataStructure;
using Amplitude.AI.Core.Framework;
using Amplitude.AI.Core.Modules;
using Amplitude.AI.Core.Services;
using Amplitude.AI.Core.Snapshots;
using Amplitude.AI.Core.Snapshots.ContextData;
using Amplitude.AI.Core.Snapshots.DatabaseSnapshots;
using Amplitude.AI.Core.Snapshots.EntitySnapshots;
using Amplitude.AI.Core.Snapshots.EntitySnapshots.Colony;
using Amplitude.AI.Core.Snapshots.EntitySnapshots.Empire;
using Amplitude.AI.Core.Snapshots.EntitySnapshots.ShipDesign;

namespace Amplitude.AI.Core.Brain.Strategic.Jobs.GlobalMilitary
{
    public class ComputeShipDesignComposition : StrategicJob
    {
        public ComputeShipDesignComposition()
        {
            base.WaitForOtherJob.Add(typeof(ComputeTaskForceComposition));
            base.WaitForOtherJob.Add(typeof(ComputeBattleData));
            base.WaitForOtherJob.Add(typeof(ComputeBestEnemy));
            this.sortByScoreDelegate = new Comparison<ModuleSnapshot>(this.SortByScore);
        }

        public override void Bind(EmpireSnapshot empire)
        {
            base.Bind(empire);
            this.majorEmpireSnapshot = (base.EmpireSnapshot as MajorEmpireSnapshot);
        }

        public ComputeShipDesignComposition.ShipdesignInfo GetDesign(string hull)
        {
            ComputeShipDesignComposition.ShipdesignInfo shipdesignInfo = null;
            for (int i = 0; i < this.designPool.Count; i++)
            {
                ComputeShipDesignComposition.ShipdesignInfo shipdesignInfo2 = this.designPool[i];
                if (shipdesignInfo2.Hull.Name == hull)
                {
                    shipdesignInfo = shipdesignInfo2;
                    break;
                }
            }
            if (shipdesignInfo == null)
            {
                shipdesignInfo = new ComputeShipDesignComposition.ShipdesignInfo(this);
                this.designPool.Add(shipdesignInfo);
            }
            return shipdesignInfo;
        }

        public ComputeShipDesignComposition.ShipdesignInfo GetDesign(float commandPoint, ShipPattern.ShipRole role)
        {
            ComputeShipDesignComposition.ShipdesignInfo shipdesignInfo = null;
            for (int i = 0; i < this.designPool.Count; i++)
            {
                ComputeShipDesignComposition.ShipdesignInfo shipdesignInfo2 = this.designPool[i];
                if (shipdesignInfo2.CommandPoint == commandPoint && shipdesignInfo2.Rolespeciality == role)
                {
                    shipdesignInfo = shipdesignInfo2;
                    break;
                }
            }
            if (shipdesignInfo == null)
            {
                shipdesignInfo = new ComputeShipDesignComposition.ShipdesignInfo(this);
                this.designPool.Add(shipdesignInfo);
            }
            return shipdesignInfo;
        }

        protected override bool MustCompute()
        {
            return base.IsNewTurnComputation() || base.StrategicDataHolder.MilitaryData.ShipDesignPattern.Count == 0;
        }

        protected override void Compute()
        {
            FleetPattern taskForcePattern = base.StrategicDataHolder.MilitaryData.TaskForcePattern;
            IDatabaseRepositoryService service = Amplitude.AI.Core.Framework.Services.GetService<IDatabaseRepositoryService>();
            IDatabase<ModuleSnapshot> database;
            if (!service.TryGetDatabase<ModuleSnapshot>(out database))
            {
                return;
            }
            IDatabase<HullDefinitionSnapshot> database2;
            if (!service.TryGetDatabase<HullDefinitionSnapshot>(out database2))
            {
                return;
            }
            this.ComputeBestEnemySituation();
            this.OrderModuleByScore(database);
            float fleetCommandPoint = taskForcePattern.FleetCommandPoint;
            ISnapshotRepositoryService service2 = Amplitude.AI.Core.Framework.Services.GetService<ISnapshotRepositoryService>();
            for (int i = 0; i < service2.GalaxySnapshot.StrategicNames.Length; i++)
            {
                string resourceName = service2.GalaxySnapshot.StrategicNames[i];
                float strategicResourceNet = base.EmpireSnapshot.GetStrategicResourceNet(resourceName);
                this.strategicStock[i] = strategicResourceNet * base.StrategicDataHolder.MilitaryData.TaskForceMaxTurnToBuild.Value;
                this.strategicStock[i] += ((strategicResourceNet == 0f) ? 0f : base.EmpireSnapshot.GetStrategicResourceStock(resourceName));
            }
            ColonizedStarSystemSnapshot firstColony = base.EmpireSnapshot.GetFirstColony();
            if (firstColony == null)
            {
                return;
            }
            List<ShipDesignData> shipDesigns = firstColony.StarSystemNode.GetAnalysisData<NodeData>(base.EmpireIndex).ShipDesigns;
            this.DefineTaskForceShipDesigns(taskForcePattern, shipDesigns, database2);
            this.RemoveHullStrategicCost(service2);
            this.DetermineTaskForceInvisibility();
            for (int j = 0; j < this.designs.Count; j++)
            {
                ComputeShipDesignComposition.ShipdesignInfo shipdesignInfo = this.designs[j];
                shipdesignInfo.ComputeStock(this.strategicStock, fleetCommandPoint, base.EmpireSnapshot);
                shipdesignInfo.ComputeSlotCategories();
                shipdesignInfo.ComputeSlotTypes();
                shipdesignInfo.ComputeModules(database, this.majorEmpireSnapshot, true);
                base.StrategicDataHolder.MilitaryData.UpdateShipDesignPattern(shipdesignInfo.SlotNames, shipdesignInfo.Model, shipdesignInfo.CommandPoint, shipdesignInfo.Rolespeciality);
            }
            for (int k = 0; k < this.strategicStock.Length; k++)
            {
                this.strategicStock[k] = 0f;
            }
            for (int l = 0; l < ComputeShipDesignComposition.specializedRole.Length; l++)
            {
                for (int m = 0; m < this.strategicStock.Length; m++)
                {
                    string resourceName2 = service2.GalaxySnapshot.StrategicNames[m];
                    float num = base.EmpireSnapshot.GetStrategicResourceNet(resourceName2);
                    ShipPattern.ShipRole shipRole = ComputeShipDesignComposition.specializedRole[l];
                    if (shipRole == ShipPattern.ShipRole.Battleship || shipRole == ShipPattern.ShipRole.MilitaryJuggernaut || shipRole == ShipPattern.ShipRole.Obliterator)
                    {
                        num *= 3f;
                    }
                    this.strategicStock[m] = num;
                }
                ShipPattern.ShipRole shipRole2 = ComputeShipDesignComposition.specializedRole[l];
                ShipDesignSnapshot targetShipDesignSnapshot = this.GetTargetShipDesignSnapshot(shipDesigns, shipRole2);
                if (targetShipDesignSnapshot != null)
                {
                    ShipDesignLayoutContextData layoutContextData = this.majorEmpireSnapshot.GetLayoutContextData(targetShipDesignSnapshot.Hull);
                    HullDefinitionSnapshot hull;
                    if (layoutContextData == null)
                    {
                        Diagnostics.LogWarning("Cant find layout or module for {0}", shipRole2);
                    }
                    else if (!database2.TryGetValue(layoutContextData.HullName, out hull))
                    {
                        Diagnostics.LogWarning("Empire {0} : Cant find hull for {1}", base.EmpireSnapshot.EmpireIndex, layoutContextData.HullName);
                    }
                    else
                    {
                        ComputeShipDesignComposition.ShipdesignInfo design = this.GetDesign(targetShipDesignSnapshot.CommandPoints, shipRole2);
                        design.Init(targetShipDesignSnapshot, layoutContextData, hull);
                        design.Number = 1;
                        design.ComputeStock(this.strategicStock, targetShipDesignSnapshot.CommandPoints, base.EmpireSnapshot);
                        design.ComputeSlotCategories();
                        design.ComputeSlotTypes();
                        design.ComputeModules(database, this.majorEmpireSnapshot, false);
                        base.StrategicDataHolder.MilitaryData.UpdateShipDesignPattern(design.SlotNames, targetShipDesignSnapshot.Model, design.CommandPoint, design.Rolespeciality);
                    }
                }
            }
            for (int n = 0; n < this.strategicStock.Length; n++)
            {
                string resourceName3 = service2.GalaxySnapshot.StrategicNames[n];
                float strategicResourceNet2 = base.EmpireSnapshot.GetStrategicResourceNet(resourceName3);
                this.strategicStock[n] = strategicResourceNet2;
            }
            for (int num2 = 0; num2 < base.EmpireSnapshot.LastestShipDesignSnapshots.Count; num2++)
            {
                ShipDesignSnapshot shipDesignSnapshot = base.EmpireSnapshot.LastestShipDesignSnapshots[num2];
                if (shipDesignSnapshot != null && shipDesignSnapshot.ShipRole == ShipPattern.ShipRole.Hero)
                {
                    ShipDesignLayoutContextData layoutContextData2 = this.majorEmpireSnapshot.GetLayoutContextData(shipDesignSnapshot.Hull);
                    HullDefinitionSnapshot hull2;
                    if (layoutContextData2 == null)
                    {
                        Diagnostics.LogWarning("Cant find layout or module for {0}", shipDesignSnapshot.Role);
                    }
                    else if (!database2.TryGetValue(layoutContextData2.HullName, out hull2))
                    {
                        Diagnostics.LogWarning("Empire {0} : Cant find hull for {1}", base.EmpireSnapshot.EmpireIndex, layoutContextData2.HullName);
                    }
                    else
                    {
                        ComputeShipDesignComposition.ShipdesignInfo design2 = this.GetDesign(shipDesignSnapshot.Hull);
                        design2.Init(shipDesignSnapshot, layoutContextData2, hull2);
                        design2.Number = 1;
                        design2.ComputeStock(this.strategicStock, 1f, base.EmpireSnapshot);
                        design2.ComputeSlotCategories();
                        design2.ComputeSlotTypes();
                        design2.ComputeModules(database, this.majorEmpireSnapshot, false);
                        base.StrategicDataHolder.MilitaryData.UpdateShipDesignPattern(design2.SlotNames, shipDesignSnapshot.Model, design2.CommandPoint, design2.Rolespeciality);
                    }
                }
            }
        }

        private void DefineTaskForceShipDesigns(FleetPattern taskForcePattern, List<ShipDesignData> shipDesignSnapshots, IDatabase<HullDefinitionSnapshot> hullDatabase)
        {
            this.designs.Clear();
            this.AddDesign(shipDesignSnapshots, hullDatabase, ShipPattern.ShipRole.Attacker, 1f);
            this.AddDesign(shipDesignSnapshots, hullDatabase, ShipPattern.ShipRole.Attacker, 3f);
            this.AddDesign(shipDesignSnapshots, hullDatabase, ShipPattern.ShipRole.Support, 1f);
            this.AddDesign(shipDesignSnapshots, hullDatabase, ShipPattern.ShipRole.Support, 3f);
            this.AddDesign(shipDesignSnapshots, hullDatabase, ShipPattern.ShipRole.Carrier, 6f);
            for (int i = 0; i < taskForcePattern.ShipRoles.Count; i++)
            {
                ShipPattern shipPattern = taskForcePattern.ShipRoles[i];
                ComputeShipDesignComposition.ShipdesignInfo currentShipdesign = this.GetCurrentShipdesign(shipPattern.RoleSpeciality, shipPattern.CommandPoint);
                if (currentShipdesign != null)
                {
                    ComputeShipDesignComposition.ShipdesignInfo shipdesignInfo = currentShipdesign;
                    int number = shipdesignInfo.Number + 1;
                    shipdesignInfo.Number = number;
                }
            }
        }

        private void DetermineTaskForceInvisibility()
        {
            bool useInvisibility = base.StrategicDataHolder.MilitaryData.AimedInvisibilityLevel > 0;
            bool useDetection = base.StrategicDataHolder.MilitaryData.AimedDetectionLevel > 0;
            ComputeShipDesignComposition.ShipdesignInfo shipdesignInfo = null;
            for (int i = 0; i < this.designs.Count; i++)
            {
                ComputeShipDesignComposition.ShipdesignInfo shipdesignInfo2 = this.designs[i];
                this.ComputeInvisibilityDesign(ref shipdesignInfo2, ref shipdesignInfo, useDetection, useInvisibility);
            }
            shipdesignInfo = null;
            for (int j = 0; j < this.designPool.Count; j++)
            {
                ComputeShipDesignComposition.ShipdesignInfo shipdesignInfo3 = this.designPool[j];
                this.ComputeInvisibilityDesign(ref shipdesignInfo3, ref shipdesignInfo, useDetection, useInvisibility);
            }
        }

        private void ComputeInvisibilityDesign(ref ComputeShipDesignComposition.ShipdesignInfo design, ref ComputeShipDesignComposition.ShipdesignInfo biggestDesign, bool useDetection, bool useInvisibility)
        {
            if (design.Rolespeciality == ShipPattern.ShipRole.MilitaryJuggernaut || design.Rolespeciality == ShipPattern.ShipRole.Battleship)
            {
                design.UseDetection = useDetection;
                design.UseInvisibility = useInvisibility;
                return;
            }
            if ((biggestDesign == null || design.CommandPoint > biggestDesign.CommandPoint) && (design.Rolespeciality == ShipPattern.ShipRole.Carrier || design.Rolespeciality == ShipPattern.ShipRole.Support))
            {
                design.UseDetection = useDetection;
                design.UseInvisibility = useInvisibility;
                if (biggestDesign != null)
                {
                    biggestDesign.UseDetection = false;
                    biggestDesign.UseInvisibility = false;
                }
                biggestDesign = design;
            }
        }

        private void AddDesign(List<ShipDesignData> shipDesignSnapshots, IDatabase<HullDefinitionSnapshot> hullDatabase, ShipPattern.ShipRole role, float commandPoint)
        {
            if (this.GetCurrentShipdesign(role, commandPoint) == null)
            {
                ShipDesignSnapshot targetShipDesignSnapshot = this.GetTargetShipDesignSnapshot(shipDesignSnapshots, commandPoint, role);
                if (targetShipDesignSnapshot == null)
                {
                    return;
                }
                ShipDesignLayoutContextData layoutContextData = this.majorEmpireSnapshot.GetLayoutContextData(targetShipDesignSnapshot.Hull);
                if (layoutContextData == null)
                {
                    Diagnostics.LogWarning("Empire {0} : Cant find Layout for {1}{2}", base.EmpireSnapshot.EmpireIndex, commandPoint, role);
                    return;
                }
                ComputeShipDesignComposition.ShipdesignInfo design = this.GetDesign(commandPoint, role);
                HullDefinitionSnapshot hull;
                if (!hullDatabase.TryGetValue(layoutContextData.HullName, out hull))
                {
                    Diagnostics.LogWarning("Empire {0} : Cant find hull for {1}", base.EmpireSnapshot.EmpireIndex, layoutContextData.HullName);
                    return;
                }
                design.Init(targetShipDesignSnapshot, layoutContextData, hull);
                this.designs.Add(design);
            }
        }

        private void RemoveHullStrategicCost(ISnapshotRepositoryService snapshotRepositoryService)
        {
            for (int i = 0; i < this.designs.Count; i++)
            {
                ComputeShipDesignComposition.ShipdesignInfo shipdesignInfo = this.designs[i];
                ConstructibleContextData constructibleContextData = null;
                for (int j = 0; j < this.majorEmpireSnapshot.HullContextCostData.Count; j++)
                {
                    ConstructibleContextData constructibleContextData2 = this.majorEmpireSnapshot.HullContextCostData[j];
                    if (constructibleContextData2.ElementKey == shipdesignInfo.Hull.Name)
                    {
                        constructibleContextData = constructibleContextData2;
                        break;
                    }
                }
                for (int k = 0; k < constructibleContextData.CostsSnapshot.Costs.Length; k++)
                {
                    CostSnapshot costSnapshot = constructibleContextData.CostsSnapshot.Costs[k];
                    int num = Array.IndexOf<string>(snapshotRepositoryService.GalaxySnapshot.StrategicNames, costSnapshot.ResourceName);
                    if (num >= 0)
                    {
                        this.strategicStock[num] = System.Math.Max(0f, this.strategicStock[num] - costSnapshot.Amount);
                    }
                }
            }
        }

        protected void OrderModuleByScore(IDatabase<ModuleSnapshot> moduleDatabase)
        {
            this.modulesOrdered.Clear();
            for (int i = 0; i < this.bestEnemySituation.ModuleSituations.Count; i++)
            {
                BattleSituationData battleSituationData = this.bestEnemySituation.ModuleSituations[i];
                ModuleSnapshot item;
                moduleDatabase.TryGetValue(battleSituationData.Name, out item);
                this.modulesOrdered.Add(item);
            }
            this.modulesOrdered.Sort(this.sortByScoreDelegate);
        }

        protected bool HasStrategicCost(ModuleSnapshot module)
        {
            List<ModuleContextData> moduleContextData = this.majorEmpireSnapshot.ModuleContextData;
            int count = moduleContextData.Count;
            for (int i = 0; i < count; i++)
            {
                ModuleContextData moduleContextData2 = moduleContextData[i];
                if (moduleContextData2.ElementKey == module.Name)
                {
                    return moduleContextData2.HasStrategicCost;
                }
            }
            return false;
        }

        protected int SortByScore(ModuleSnapshot left, ModuleSnapshot right)
        {
            BattleSituationData battleSituationData = null;
            List<BattleSituationData> moduleSituations = this.bestEnemySituation.ModuleSituations;
            int count = moduleSituations.Count;
            for (int i = 0; i < count; i++)
            {
                BattleSituationData battleSituationData2 = moduleSituations[i];
                if (battleSituationData2.Name == left.Name)
                {
                    battleSituationData = battleSituationData2;
                    break;
                }
            }
            BattleSituationData battleSituationData3 = null;
            for (int j = 0; j < count; j++)
            {
                BattleSituationData battleSituationData4 = moduleSituations[j];
                if (battleSituationData4.Name == right.Name)
                {
                    battleSituationData3 = battleSituationData4;
                    break;
                }
            }
            if (battleSituationData.Value != battleSituationData3.Value)
            {
                return -1 * battleSituationData.Value.CompareTo(battleSituationData3.Value);
            }
            if (left.Level != right.Level)
            {
                return -1 * left.Level.CompareTo(right.Level);
            }
            bool flag = this.HasStrategicCost(left);
            bool flag2 = this.HasStrategicCost(right);
            if (flag == flag2)
            {
                return 0;
            }
            if (!flag)
            {
                return 1;
            }
            return -1;
        }

        protected void ComputeBestEnemySituation()
        {
            this.bestEnemySituation = null;
            for (int i = 0; i < base.StrategicDataHolder.MilitaryData.BattleSituations.Count; i++)
            {
                BattleSituationsData battleSituationsData = base.StrategicDataHolder.MilitaryData.BattleSituations[i];
                if (battleSituationsData.EmpireIndex == base.StrategicDataHolder.MilitaryData.BestEnemyIndex)
                {
                    this.bestEnemySituation = battleSituationsData;
                    break;
                }
            }
            if (this.bestEnemySituation == null)
            {
                Diagnostics.LogWarning("Can't find situation for enemy {0}", base.StrategicDataHolder.MilitaryData.BestEnemyIndex);
                return;
            }
        }

        protected ShipDesignSnapshot GetTargetShipDesignSnapshot(List<ShipDesignData> snapshots, float commandPoint, ShipPattern.ShipRole shipRole)
        {
            for (int i = 0; i < snapshots.Count; i++)
            {
                ShipDesignSnapshot shipDesignSnapshot = snapshots[i].ShipDesignSnapshot;
                if (shipDesignSnapshot.ShipRole == shipRole && shipDesignSnapshot.CommandPoints == commandPoint)
                {
                    return shipDesignSnapshot;
                }
            }
            return null;
        }

        protected ShipDesignSnapshot GetTargetShipDesignSnapshot(List<ShipDesignData> snapshots, ShipPattern.ShipRole shipRole)
        {
            for (int i = 0; i < snapshots.Count; i++)
            {
                ShipDesignSnapshot shipDesignSnapshot = snapshots[i].ShipDesignSnapshot;
                if (shipDesignSnapshot.ShipRole == shipRole)
                {
                    return shipDesignSnapshot;
                }
            }
            return null;
        }

        private ComputeShipDesignComposition.ShipdesignInfo GetCurrentShipdesign(ShipPattern.ShipRole role, float commandPoint)
        {
            ComputeShipDesignComposition.ShipdesignInfo result = null;
            for (int i = 0; i < this.designs.Count; i++)
            {
                ComputeShipDesignComposition.ShipdesignInfo shipdesignInfo = this.designs[i];
                if (shipdesignInfo.CommandPoint == commandPoint && shipdesignInfo.Rolespeciality == role)
                {
                    result = shipdesignInfo;
                    break;
                }
            }
            return result;
        }

        static ComputeShipDesignComposition()
        {
        }

        private static ShipPattern.ShipRole[] specializedRole = new ShipPattern.ShipRole[]
		{
			ShipPattern.ShipRole.Colonizer,
			ShipPattern.ShipRole.Exploration,
			ShipPattern.ShipRole.Leecher,
			ShipPattern.ShipRole.MotherShip,
			ShipPattern.ShipRole.Invader,
			ShipPattern.ShipRole.RootCreator,
			ShipPattern.ShipRole.PlanetDestroyer,
			ShipPattern.ShipRole.MilitaryJuggernaut,
			ShipPattern.ShipRole.ScienceJuggernaut,
			ShipPattern.ShipRole.SupportJuggernaut,
			ShipPattern.ShipRole.Battleship,
			ShipPattern.ShipRole.Obliterator
		};

        private float[] strategicStock = new float[6];

        private List<ComputeShipDesignComposition.ShipdesignInfo> designPool = new List<ComputeShipDesignComposition.ShipdesignInfo>();

        private List<ComputeShipDesignComposition.ShipdesignInfo> designs = new List<ComputeShipDesignComposition.ShipdesignInfo>();

        private List<ModuleSnapshot> modulesOrdered = new List<ModuleSnapshot>();

        private BattleSituationsData bestEnemySituation;

        private Comparison<ModuleSnapshot> sortByScoreDelegate;

        private MajorEmpireSnapshot majorEmpireSnapshot;

        public class ShipdesignInfo
        {
            public ShipdesignInfo(ComputeShipDesignComposition computeShipDesignComposition)
            {
                this.computeShipDesignComposition = computeShipDesignComposition;
            }

            public int Number { get; set; }

            public uint Model { get; set; }

            public float CommandPoint { get; set; }

            public bool UseDetection { get; set; }

            public bool UseInvisibility { get; set; }

            public ShipPattern.ShipRole Rolespeciality { get; set; }

            public ShipDesignSnapshot Snapshot { get; set; }

            public ShipDesignLayoutContextData Layout { get; set; }

            public HullDefinitionSnapshot Hull { get; set; }

            public float[] CostMultiplier { get; set; }

            public string[] SlotNames { get; set; }

            public ShipLayoutSlotSnapshot.ModuleCategory[] SlotCategories { get; set; }

            public string[][] SlotTypes { get; set; }

            public float[] StrategicStock { get; set; }

            public List<int> IndexOrder { get; set; }

            public int WeaponCount { get; set; }

            public int SquadronCount { get; set; }

            public string MainWeaponFamily { get; set; }

            public int WeaponSlotCount
            {
                get
                {
                    int num = 0;
                    for (int i = 0; i < this.SlotCategories.Length; i++)
                    {
                        if (this.SlotContainsCategory(i, ShipLayoutSlotSnapshot.ModuleCategory.Weapon))
                        {
                            num++;
                        }
                    }
                    return num;
                }
            }

            public int SquadronSlotCount
            {
                get
                {
                    int num = 0;
                    for (int i = 0; i < this.SlotCategories.Length; i++)
                    {
                        if (this.SlotContainsCategory(i, ShipLayoutSlotSnapshot.ModuleCategory.Squadron))
                        {
                            num++;
                        }
                    }
                    return num;
                }
            }

            public bool SlotContainsCategory(int index, ShipLayoutSlotSnapshot.ModuleCategory category)
            {
                return (this.SlotCategories[index] & category) == category;
            }

            public void Init(ShipDesignSnapshot snapshot, ShipDesignLayoutContextData layout, HullDefinitionSnapshot hull)
            {
                this.Model = snapshot.Model;
                this.CommandPoint = snapshot.CommandPoints;
                this.Rolespeciality = snapshot.ShipRole;
                this.Snapshot = snapshot;
                this.Layout = layout;
                this.Hull = hull;
                this.Number = 0;
                if (this.StrategicStock == null)
                {
                    this.StrategicStock = new float[6];
                }
                if (this.SlotCategories == null || this.SlotCategories.Length != layout.Slots.Length)
                {
                    this.SlotCategories = new ShipLayoutSlotSnapshot.ModuleCategory[layout.Slots.Length];
                }
                if (this.SlotTypes == null || this.SlotTypes.Length != layout.Slots.Length)
                {
                    this.SlotTypes = new string[layout.Slots.Length][];
                    for (int i = 0; i < layout.Slots.Length; i++)
                    {
                        this.SlotTypes[i] = null;
                    }
                }
                if (this.SlotNames == null || this.SlotNames.Length != layout.Slots.Length)
                {
                    this.SlotNames = new string[layout.Slots.Length];
                }
                if (this.CostMultiplier == null || this.CostMultiplier.Length != layout.Slots.Length)
                {
                    this.CostMultiplier = new float[layout.Slots.Length];
                }
                if (this.IndexOrder == null)
                {
                    this.IndexOrder = new List<int>();
                }
                for (int j = 0; j < layout.Slots.Length; j++)
                {
                    this.SlotCategories[j] = ShipLayoutSlotSnapshot.ModuleCategory.None;
                    this.SlotTypes[j] = null;
                    this.SlotNames[j] = null;
                    this.CostMultiplier[j] = 1f;
                    this.CostMultiplier[j] *= hull.ModuleCostMultiplier;
                    this.CostMultiplier[j] *= layout.Slots[j].ModuleCostMultiplier;
                }
                this.IndexOrder.Clear();
            }

            public void ComputeStock(float[] strategicStock, float fleetCommandPoint, EmpireSnapshot empireSnapshot)
            {
                for (int i = 0; i < 6; i++)
                {
                    float num = this.CommandPoint;
                    if (num == 0f)
                    {
                        num = 1f;
                    }
                    this.StrategicStock[i] = strategicStock[i] * num * (float)this.Number / fleetCommandPoint;
                }
                ISnapshotRepositoryService service = Amplitude.AI.Core.Framework.Services.GetService<ISnapshotRepositoryService>();
                for (int j = 0; j < this.Layout.Slots.Length; j++)
                {
                    ShipLayoutSlotSnapshot shipLayoutSlotSnapshot = this.Layout.Slots[j];
                    if (shipLayoutSlotSnapshot.Costs.Costs != null)
                    {
                        for (int k = 0; k < shipLayoutSlotSnapshot.Costs.Costs.Length; k++)
                        {
                            CostSnapshot costSnapshot = shipLayoutSlotSnapshot.Costs.Costs[k];
                            int num2 = Array.IndexOf<string>(service.GalaxySnapshot.StrategicNames, costSnapshot.ResourceName);
                            if (num2 >= 0)
                            {
                                if (this.StrategicStock[num2] >= costSnapshot.Amount * (float)this.Number)
                                {
                                    this.StrategicStock[num2] -= costSnapshot.Amount * (float)this.Number;
                                }
                                else
                                {
                                    this.SlotCategories[j] = ShipLayoutSlotSnapshot.ModuleCategory.Forbidden;
                                }
                            }
                        }
                    }
                }
            }

            public void FillSlots(ShipLayoutSlotSnapshot.ModuleCategory category)
            {
                while (this.AddOneSlot(category))
                {
                }
            }

            public void FillAttackSlot(int minWeapon, bool hasSquadronModule)
            {
                if (!hasSquadronModule)
                {
                    this.FillSlots(ShipLayoutSlotSnapshot.ModuleCategory.Weapon);
                    return;
                }
                int num = 0;
                while (num < minWeapon && this.AddOneSlot(ShipLayoutSlotSnapshot.ModuleCategory.Weapon))
                {
                    num++;
                }
                bool flag;
                bool flag2;
                do
                {
                    flag = this.AddOneSlot(ShipLayoutSlotSnapshot.ModuleCategory.Squadron);
                    flag2 = this.AddOneSlot(ShipLayoutSlotSnapshot.ModuleCategory.Weapon);
                }
                while (flag || flag2);
            }

            public void ComputeSlotCategories()
            {
                bool flag = this.GetBestSquadronModule() != null;
                if (this.Rolespeciality != ShipPattern.ShipRole.PlanetDestroyer)
                {
                    this.AddOneSlot(ShipLayoutSlotSnapshot.ModuleCategory.Weapon);
                }
                if (this.Rolespeciality == ShipPattern.ShipRole.Exploration)
                {
                    this.AddOneSlot(ShipLayoutSlotSnapshot.ModuleCategory.Support);
                    this.AddOneSlot(ShipLayoutSlotSnapshot.ModuleCategory.Support);
                }
                if (this.Rolespeciality == ShipPattern.ShipRole.Exploration)
                {
                    this.AddOneSlot(ShipLayoutSlotSnapshot.ModuleCategory.Support);
                    this.AddOneSlot(ShipLayoutSlotSnapshot.ModuleCategory.Support);
                }
                if (this.Rolespeciality != ShipPattern.ShipRole.Leecher)
                {
                    this.AddOneSlot(ShipLayoutSlotSnapshot.ModuleCategory.Defense);
                }
                this.AddOneSlot(ShipLayoutSlotSnapshot.ModuleCategory.Defense);
                this.AddOneSlot(ShipLayoutSlotSnapshot.ModuleCategory.Support);
                if (this.Rolespeciality == ShipPattern.ShipRole.Attacker)
                {
                    int minWeapon = (this.CommandPoint == 1f) ? 2 : 0;
                    this.FillAttackSlot(minWeapon, flag);
                    this.AddOneSlot(ShipLayoutSlotSnapshot.ModuleCategory.Defense);
                    this.AddOneSlot(ShipLayoutSlotSnapshot.ModuleCategory.Support);
                    this.FillSlots(ShipLayoutSlotSnapshot.ModuleCategory.Defense);
                    this.FillSlots(ShipLayoutSlotSnapshot.ModuleCategory.Support);
                    return;
                }
                if (this.Rolespeciality == ShipPattern.ShipRole.Carrier)
                {
                    if (flag)
                    {
                        this.FillSlots(ShipLayoutSlotSnapshot.ModuleCategory.Squadron);
                        this.AddOneSlot(ShipLayoutSlotSnapshot.ModuleCategory.Weapon);
                    }
                    else
                    {
                        this.FillSlots(ShipLayoutSlotSnapshot.ModuleCategory.Weapon);
                    }
                    this.FillSlots(ShipLayoutSlotSnapshot.ModuleCategory.Defense);
                    this.FillSlots(ShipLayoutSlotSnapshot.ModuleCategory.Support);
                    this.FillSlots(ShipLayoutSlotSnapshot.ModuleCategory.Weapon);
                    return;
                }
                if (this.Rolespeciality == ShipPattern.ShipRole.PlanetDestroyer)
                {
                    this.FillSlots(ShipLayoutSlotSnapshot.ModuleCategory.Destruction);
                    this.FillSlots(ShipLayoutSlotSnapshot.ModuleCategory.Defense);
                    this.FillSlots(ShipLayoutSlotSnapshot.ModuleCategory.Support);
                    return;
                }
                if (this.Rolespeciality == ShipPattern.ShipRole.Support)
                {
                    if (flag)
                    {
                        this.FillSlots(ShipLayoutSlotSnapshot.ModuleCategory.Squadron);
                    }
                    this.AddOneSlot(ShipLayoutSlotSnapshot.ModuleCategory.Support);
                    this.AddOneSlot(ShipLayoutSlotSnapshot.ModuleCategory.Defense);
                    this.FillSlots(ShipLayoutSlotSnapshot.ModuleCategory.Support);
                    this.FillSlots(ShipLayoutSlotSnapshot.ModuleCategory.Defense);
                    this.FillAttackSlot(0, flag);
                    return;
                }
                if (this.Rolespeciality == ShipPattern.ShipRole.SupportJuggernaut)
                {
                    this.FillSlots(ShipLayoutSlotSnapshot.ModuleCategory.Support);
                    this.FillSlots(ShipLayoutSlotSnapshot.ModuleCategory.Defense);
                    this.FillSlots(ShipLayoutSlotSnapshot.ModuleCategory.Weapon);
                    return;
                }
                if (this.Rolespeciality == ShipPattern.ShipRole.ScienceJuggernaut)
                {
                    this.FillSlots(ShipLayoutSlotSnapshot.ModuleCategory.Support);
                    this.FillSlots(ShipLayoutSlotSnapshot.ModuleCategory.Defense);
                    this.FillSlots(ShipLayoutSlotSnapshot.ModuleCategory.Weapon);
                    return;
                }
                if (this.Rolespeciality == ShipPattern.ShipRole.MilitaryJuggernaut || this.Rolespeciality == ShipPattern.ShipRole.Battleship)
                {
                    this.FillAttackSlot(0, flag);
                    this.FillSlots(ShipLayoutSlotSnapshot.ModuleCategory.Support);
                    this.FillSlots(ShipLayoutSlotSnapshot.ModuleCategory.Defense);
                    return;
                }
                if (this.Rolespeciality == ShipPattern.ShipRole.Obliterator)
                {
                    this.FillSlots(ShipLayoutSlotSnapshot.ModuleCategory.Defense);
                    this.FillSlots(ShipLayoutSlotSnapshot.ModuleCategory.Support);
                    this.FillSlots(ShipLayoutSlotSnapshot.ModuleCategory.Weapon);
                    return;
                }
                this.FillSlots(ShipLayoutSlotSnapshot.ModuleCategory.Support);
                this.FillSlots(ShipLayoutSlotSnapshot.ModuleCategory.Defense);
                this.FillSlots(ShipLayoutSlotSnapshot.ModuleCategory.Weapon);
            }

            public bool AddOneSlot(ShipLayoutSlotSnapshot.ModuleCategory category)
            {
                for (int i = 0; i < this.SlotCategories.Length; i++)
                {
                    if (i < this.Layout.Slots.Length && i >= 0)
                    {
                        ShipLayoutSlotSnapshot shipLayoutSlotSnapshot = this.Layout.Slots[i];
                        if (this.SlotCategories[i] == ShipLayoutSlotSnapshot.ModuleCategory.None && shipLayoutSlotSnapshot.ArePrerequisitesValid && !shipLayoutSlotSnapshot.IsHidden && shipLayoutSlotSnapshot.Categories.Length == 1 && shipLayoutSlotSnapshot.ContainsCategory(category))
                        {
                            this.SlotCategories[i] = category;
                            this.IndexOrder.Add(i);
                            return true;
                        }
                    }
                }
                for (int j = 0; j < this.SlotCategories.Length; j++)
                {
                    if (j < this.Layout.Slots.Length && j >= 0)
                    {
                        ShipLayoutSlotSnapshot shipLayoutSlotSnapshot2 = this.Layout.Slots[j];
                        if (this.SlotCategories[j] == ShipLayoutSlotSnapshot.ModuleCategory.None && shipLayoutSlotSnapshot2.ArePrerequisitesValid && !shipLayoutSlotSnapshot2.IsHidden && shipLayoutSlotSnapshot2.ContainsCategory(category))
                        {
                            this.SlotCategories[j] = category;
                            this.IndexOrder.Add(j);
                            return true;
                        }
                    }
                }
                return false;
            }

            public void ComputeSlotTypes()
            {
                this.ComputeDestroyerSlotTypes();
                this.ComputeSupportSlotTypes();
                this.ComputeSquadronSlotTypes();
            }

            public void ComputeSquadronSlotTypes()
            {
                bool flag = this.computeShipDesignComposition.bestEnemySituation.EmpireSituation.UseSituationValues[12] > 0f;
                ShipLayoutSlotSnapshot.ModuleCategory category = ShipLayoutSlotSnapshot.ModuleCategory.Squadron;
                int num = 0;
                for (int i = 0; i < this.SlotCategories.Length; i++)
                {
                    if (this.SlotContainsCategory(i, category))
                    {
                        num++;
                    }
                }
                if (!flag)
                {
                    this.FillSlotType(category, this.bomber, num);
                    return;
                }
                int num2 = num / 2;
                this.FillSlotType(category, this.bomber, num2);
                this.FillSlotType(category, this.fighter, num - num2);
            }

            public void ComputeDestroyerSlotTypes()
            {
                ShipLayoutSlotSnapshot.ModuleCategory category = ShipLayoutSlotSnapshot.ModuleCategory.Destruction;
                int num = 0;
                for (int i = 0; i < this.SlotCategories.Length; i++)
                {
                    if (this.SlotContainsCategory(i, category))
                    {
                        num++;
                    }
                }
                this.FillSlotType(category, this.destroyerAttack, num);
            }

            public void ComputeSupportSlotTypes()
            {
                bool flag = false;
                bool flag2 = false;
                WeaponModuleSnapshot bestWeaponModule = this.GetBestWeaponModule();
                if (bestWeaponModule != null)
                {
                    flag = (bestWeaponModule.BattleSituation.UseSituationValues[3].Value > 0f);
                }
                DefenseModuleSnapshot bestDefenseModule = this.GetBestDefenseModule();
                if (bestDefenseModule != null)
                {
                    flag2 = (bestDefenseModule.BattleSituation.UseSituationValues[13].Value > 0f);
                }
                ShipLayoutSlotSnapshot.ModuleCategory category = ShipLayoutSlotSnapshot.ModuleCategory.Support;
                int num = 0;
                for (int i = 0; i < this.SlotCategories.Length; i++)
                {
                    if (this.SlotContainsCategory(i, category))
                    {
                        num++;
                    }
                }
                if (this.Rolespeciality == ShipPattern.ShipRole.Attacker)
                {
                    int num2 = 1;
                    int count = num - num2;
                    this.FillSlotType(category, this.engineSupport, num2);
                    this.FillSlotType(category, flag ? this.energyAttackerSupport : this.projectileAttackerSupport, count);
                    return;
                }
                if (this.Rolespeciality == ShipPattern.ShipRole.Support)
                {
                    int num3 = 1;
                    int num4 = num - num3;
                    int count2 = 0;
                    int count3 = 0;
                    if (num4 > 0 && this.UseDetection)
                    {
                        count3 = 1;
                        num4--;
                    }
                    if (num4 > 0 && this.UseInvisibility)
                    {
                        count2 = 1;
                        num4--;
                    }
                    this.FillSlotType(category, this.detectionSupport, count3);
                    this.FillSlotType(category, this.invisibilitySupport, count2);
                    this.FillSlotType(category, this.engineSupport, num3);
                    this.FillSlotType(category, flag2 ? this.defenserShieldSupport : this.defenserHullSupport, num4);
                    return;
                }
                if (this.Rolespeciality == ShipPattern.ShipRole.Carrier || this.Rolespeciality == ShipPattern.ShipRole.MilitaryJuggernaut || this.Rolespeciality == ShipPattern.ShipRole.Battleship)
                {
                    bool flag4 = this.HaveModuleFamilly("TractorBeam");
                    int num5 = 1;
                    int num6 = flag4 ? 1 : 0;
                    int num7 = num - num5 - num6;
                    int count4 = 0;
                    int count5 = 0;
                    if (num7 > 0 && this.UseInvisibility)
                    {
                        count4 = 1;
                        num7--;
                    }
                    if (num7 > 0 && this.UseDetection)
                    {
                        count5 = 1;
                        num7--;
                    }
                    this.FillSlotType(category, this.invisibilitySupport, count4);
                    this.FillSlotType(category, this.detectionSupport, count5);
                    this.FillSlotType(category, this.engineSupport, num5);
                    this.FillSlotType(category, this.tractorBeam, num6);
                    this.FillSlotType(category, flag ? this.energyAttackerSupport : this.projectileAttackerSupport, num7);
                    return;
                }
                if (this.Rolespeciality == ShipPattern.ShipRole.Obliterator)
                {
                    int num8 = 1;
                    int count6 = num - num8;
                    this.FillSlotType(category, this.engineSupport, num8);
                    this.FillSlotType(category, flag ? this.energyAttackerSupport : this.projectileAttackerSupport, count6);
                    return;
                }
                if (this.Rolespeciality == ShipPattern.ShipRole.PlanetDestroyer)
                {
                    int num9 = 1;
                    int count7 = num - num9;
                    this.FillSlotType(category, this.engineSupport, num9);
                    this.FillSlotType(category, flag2 ? this.defenserShieldSupport : this.defenserHullSupport, count7);
                    return;
                }
                if (this.Rolespeciality == ShipPattern.ShipRole.MotherShip)
                {
                    int count8 = (num - 2 > 2) ? 2 : (num - 2);
                    int count9 = (num - 4 > 1) ? 1 : (num - 4);
                    int count10 = (num - 5 > 1) ? 1 : (num - 5);
                    int count11 = num - 6;
                    this.FillSlotType(category, this.motherShipAnchored, 1);
                    this.FillSlotType(category, this.engineSupport, 1);
                    this.FillSlotType(category, this.motherShipSupport, count8);
                    this.FillSlotType(category, this.motherShipIndustry, count9);
                    this.FillSlotType(category, this.motherShipDust, count10);
                    this.FillSlotType(category, this.motherShipScience, count11);
                    return;
                }
                if (this.Rolespeciality == ShipPattern.ShipRole.Colonizer)
                {
                    int num10 = 1;
                    int count12 = num - num10;
                    this.FillSlotType(category, this.engineSupport, count12);
                    this.FillSlotType(category, this.colonizerSupport, num10);
                    return;
                }
                if (this.Rolespeciality == ShipPattern.ShipRole.Exploration)
                {
                    int num11 = num / 2;
                    int count13 = num - num11;
                    this.FillSlotType(category, this.engineSupport, num11);
                    this.FillSlotType(category, this.explorerSupport, count13);
                    return;
                }
                if (this.Rolespeciality == ShipPattern.ShipRole.Leecher)
                {
                    this.FillSlotType(category, this.engineSupport, 1);
                    this.FillSlotType(category, this.leecherSupport, num - 1);
                    return;
                }
                if (this.Rolespeciality == ShipPattern.ShipRole.Invader)
                {
                    int num12 = 1;
                    int num13 = 1;
                    int count14 = num - num12 - num13;
                    this.FillSlotType(category, this.engineSupport, num12);
                    this.FillSlotType(category, this.invaderSupport, num13);
                    this.FillSlotType(category, this.siegeSupport, count14);
                    return;
                }
                if (this.Rolespeciality == ShipPattern.ShipRole.RootCreator)
                {
                    int num14 = 1;
                    this.FillSlotType(category, this.rootSupport, num14);
                    int count15 = num - num14;
                    this.FillSlotType(category, this.engineSupport, count15);
                    return;
                }
                if (this.Rolespeciality == ShipPattern.ShipRole.Hero)
                {
                    int num15 = 1;
                    int count16 = num - num15;
                    this.FillSlotType(category, this.engineSupport, num15);
                    this.FillSlotType(category, flag2 ? this.defenserShieldSupport : this.defenserHullSupport, count16);
                    return;
                }
                if (this.Rolespeciality == ShipPattern.ShipRole.SupportJuggernaut)
                {
                    bool flag5 = this.HaveModuleFamilly("SystemSupportFIDS");
                    int num16 = 0;
                    int num17 = num - num16;
                    int num18 = flag5 ? (num17 - 1) : 0;
                    num17 -= num18;
                    this.FillSlotType(category, this.engineSupport, num16);
                    this.FillSlotType(category, this.juggernautFIDS, num18);
                    this.FillSlotType(category, this.juggernautMining, num17);
                    return;
                }
                if (this.Rolespeciality == ShipPattern.ShipRole.ScienceJuggernaut)
                {
                    bool flag6 = this.HaveModuleFamilly("JuggernautAnalyzer");
                    bool flag3 = this.HaveModuleFamilly("Terraformation");
                    bool flag7 = this.HaveModuleFamilly("AnomalyReduction");
                    bool flag8 = this.HaveModuleFamilly("Salvager");
                    int num19 = 0;
                    int num20 = num - num19;
                    int num21 = flag3 ? 1 : 0;
                    int count17 = flag8 ? 1 : 0;
                    int count18 = flag7 ? 1 : 0;
                    num20 -= num21;
                    int num22 = flag6 ? 1 : 0;
                    num20 -= num22;
                    this.FillSlotType(category, this.engineSupport, num19);
                    this.FillSlotType(category, this.juggernautTerraformation, num21);
                    this.FillSlotType(category, this.juggernautSalvage, count17);
                    this.FillSlotType(category, this.anomalyReduction, count18);
                    this.FillSlotType(category, this.juggernautAnalyser, num22);
                    this.FillSlotType(category, this.juggernautResearch, num20);
                }
            }

            public bool HaveModuleFamilly(string family)
            {
                for (int i = 0; i < this.computeShipDesignComposition.modulesOrdered.Count; i++)
                {
                    if (this.computeShipDesignComposition.modulesOrdered[i].Family == family)
                    {
                        return true;
                    }
                }
                return false;
            }

            public WeaponModuleSnapshot GetBestWeaponModule()
            {
                for (int i = 0; i < this.computeShipDesignComposition.modulesOrdered.Count; i++)
                {
                    WeaponModuleSnapshot weaponModuleSnapshot = this.computeShipDesignComposition.modulesOrdered[i] as WeaponModuleSnapshot;
                    if (weaponModuleSnapshot != null)
                    {
                        return weaponModuleSnapshot;
                    }
                }
                return null;
            }

            public DefenseModuleSnapshot GetBestDefenseModule()
            {
                for (int i = 0; i < this.computeShipDesignComposition.modulesOrdered.Count; i++)
                {
                    DefenseModuleSnapshot defenseModuleSnapshot = this.computeShipDesignComposition.modulesOrdered[i] as DefenseModuleSnapshot;
                    if (defenseModuleSnapshot != null)
                    {
                        return defenseModuleSnapshot;
                    }
                }
                return null;
            }

            public SquadronModuleSnapshot GetBestSquadronModule()
            {
                for (int i = 0; i < this.computeShipDesignComposition.modulesOrdered.Count; i++)
                {
                    SquadronModuleSnapshot squadronModuleSnapshot = this.computeShipDesignComposition.modulesOrdered[i] as SquadronModuleSnapshot;
                    if (squadronModuleSnapshot != null)
                    {
                        return squadronModuleSnapshot;
                    }
                }
                return null;
            }

            public void ComputeModules(IDatabase<ModuleSnapshot> database, MajorEmpireSnapshot majorEmpire, bool warship = true)
            {
                this.WeaponCount = 0;
                ModuleDecisionHelper moduleDecisionHelper = new ModuleDecisionHelper();
                int num = (this.Rolespeciality == ShipPattern.ShipRole.Attacker) ? 3 : 2;
                for (int i = 0; i < this.IndexOrder.Count; i++)
                {
                    int num2 = this.IndexOrder[i];
                    moduleDecisionHelper.setModuleCategory(this.SlotCategories[num2]);
                    if (this.SlotCategories[num2] != ShipLayoutSlotSnapshot.ModuleCategory.Forbidden)
                    {
                        ShipLayoutSlotSnapshot shipLayoutSlotSnapshot = this.Layout.Slots[num2];
                        if (this.SlotCategories[num2] != ShipLayoutSlotSnapshot.ModuleCategory.None && this.SlotCategories[num2] != ShipLayoutSlotSnapshot.ModuleCategory.Forbidden)
                        {
                            int count = 1 + shipLayoutSlotSnapshot.LinkedSlots.Length;
                            if (!this.Layout.ModuleBySlot.ContainsKey(num2))
                            {
                                Diagnostics.LogWarning(string.Format("No ModuleBySlot for index {0} type {1} for layout {2}/{3} in empire {4}", new object[]
								{
									0,
									this.SlotCategories[num2],
									this.Layout.HullName,
									this.Layout.Role,
									majorEmpire.EmpireIndex
								}));
                            }
                            else
                            {
                                bool flag = false;
                                if (this.WeaponCount >= num)
                                {
                                    flag = this.SlotContainsCategory(num2, ShipLayoutSlotSnapshot.ModuleCategory.Weapon);
                                }
                                string mainWeaponFamily = this.MainWeaponFamily;
                                List<string> list = this.Layout.ModuleBySlot[num2];
                                if (this.SlotTypes[num2] != null)
                                {
                                    bool flag2 = false;
                                    for (int j = 0; j < this.SlotTypes[num2].Length; j++)
                                    {
                                        string b = this.SlotTypes[num2][j];
                                        for (int k = 0; k < this.computeShipDesignComposition.modulesOrdered.Count; k++)
                                        {
                                            ModuleSnapshot moduleSnapshot = this.computeShipDesignComposition.modulesOrdered[k];
                                            moduleDecisionHelper.setModuleSnapshot(moduleSnapshot);
                                            if (moduleDecisionHelper.IsModuleAllowed() && list.Contains(moduleSnapshot.Name) && (this.SlotTypes[num2] == null || !(moduleSnapshot.Family != b)) && this.AddModule(moduleSnapshot, num2, majorEmpire, count))
                                            {
                                                moduleDecisionHelper.UpdateModules();
                                                flag2 = true;
                                                break;
                                            }
                                        }
                                        if (flag2)
                                        {
                                            break;
                                        }
                                    }
                                    if (!flag2)
                                    {
                                        Diagnostics.LogWarning("Could not find or pay for module of type {0} for slot {1} on ship design {2}", this.PrettyPrintArray(this.SlotTypes[num2]), num2, this.Hull.Name);
                                    }
                                }
                                else
                                {
                                    int l = 0;
                                    while (l < this.computeShipDesignComposition.modulesOrdered.Count)
                                    {
                                        ModuleSnapshot moduleSnapshot2 = this.computeShipDesignComposition.modulesOrdered[l];
                                        moduleDecisionHelper.setModuleSnapshot(moduleSnapshot2);
                                        if (moduleDecisionHelper.IsModuleAllowed() && list.Contains(moduleSnapshot2.Name) && ((this.SlotContainsCategory(num2, ShipLayoutSlotSnapshot.ModuleCategory.Weapon) && moduleSnapshot2 is WeaponModuleSnapshot) | (this.SlotContainsCategory(num2, ShipLayoutSlotSnapshot.ModuleCategory.Squadron) && moduleSnapshot2 is SquadronModuleSnapshot) | (this.SlotContainsCategory(num2, ShipLayoutSlotSnapshot.ModuleCategory.Defense) && moduleSnapshot2 is DefenseModuleSnapshot) | (this.SlotContainsCategory(num2, ShipLayoutSlotSnapshot.ModuleCategory.Support) && moduleSnapshot2 is SupportModuleSnapshot)) && (!flag || !(moduleSnapshot2.Family == mainWeaponFamily)) && this.AddModule(moduleSnapshot2, num2, majorEmpire, count))
                                        {
                                            if (!flag && this.SlotCategories[num2] == ShipLayoutSlotSnapshot.ModuleCategory.Weapon)
                                            {
                                                this.MainWeaponFamily = moduleSnapshot2.Family;
                                                break;
                                            }
                                            break;
                                        }
                                        else
                                        {
                                            l++;
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }

            public bool AddModule(ModuleSnapshot module, int slotIndex, MajorEmpireSnapshot majorEmpire, int count)
            {
                ModuleContextData moduleContextData = null;
                for (int i = 0; i < majorEmpire.ModuleContextData.Count; i++)
                {
                    ModuleContextData moduleContextData2 = majorEmpire.ModuleContextData[i];
                    if (moduleContextData2.ElementKey == module.Name)
                    {
                        moduleContextData = moduleContextData2;
                        break;
                    }
                }
                if (moduleContextData != null && moduleContextData.CostsSnapshot.Costs != null && this.CanAfford(moduleContextData, count, slotIndex))
                {
                    this.SlotNames[slotIndex] = module.Name;
                    for (int j = 0; j < this.Layout.Slots[slotIndex].LinkedSlots.Length; j++)
                    {
                        int num = this.Layout.Slots[slotIndex].LinkedSlots[j];
                        if (num < this.SlotNames.Length && num >= 0)
                        {
                            this.SlotNames[num] = module.Name;
                        }
                    }
                    this.RemoveCost(moduleContextData, count, slotIndex);
                    if (module is WeaponModuleSnapshot)
                    {
                        int num2 = this.WeaponCount + 1;
                        this.WeaponCount = num2;
                    }
                    else if (module is SquadronModuleSnapshot)
                    {
                        int num3 = this.SquadronCount + 1;
                        this.SquadronCount = num3;
                    }
                    return true;
                }
                return false;
            }

            protected void RemoveCost(ModuleContextData moduleCost, int count, int slotIndex)
            {
                ISnapshotRepositoryService service = Amplitude.AI.Core.Framework.Services.GetService<ISnapshotRepositoryService>();
                for (int i = 0; i < moduleCost.CostsSnapshot.Costs.Length; i++)
                {
                    CostSnapshot costSnapshot = moduleCost.CostsSnapshot.Costs[i];
                    int num = Array.IndexOf<string>(service.GalaxySnapshot.StrategicNames, costSnapshot.ResourceName);
                    if (num >= 0)
                    {
                        this.StrategicStock[num] -= costSnapshot.Amount * (float)count * this.CostMultiplier[slotIndex];
                    }
                }
            }

            protected bool CanAfford(ModuleContextData moduleCost, int count, int slotIndex)
            {
                ISnapshotRepositoryService service = Amplitude.AI.Core.Framework.Services.GetService<ISnapshotRepositoryService>();
                for (int i = 0; i < moduleCost.CostsSnapshot.Costs.Length; i++)
                {
                    CostSnapshot costSnapshot = moduleCost.CostsSnapshot.Costs[i];
                    int num = Array.IndexOf<string>(service.GalaxySnapshot.StrategicNames, costSnapshot.ResourceName);
                    if (num >= 0 && this.StrategicStock[num] < costSnapshot.Amount * (float)count * this.CostMultiplier[slotIndex])
                    {
                        return false;
                    }
                }
                return true;
            }

            protected void FillSlotType(ShipLayoutSlotSnapshot.ModuleCategory category, string[] moduleTypes, int count)
            {
                for (int i = 0; i < count; i++)
                {
                    for (int j = 0; j < this.SlotTypes.Length; j++)
                    {
                        if (this.SlotTypes[j] == null && this.SlotCategories[j] == category)
                        {
                            this.SlotTypes[j] = moduleTypes;
                            break;
                        }
                    }
                }
            }

            private string PrettyPrintArray(string[] array)
            {
                string text = "";
                for (int i = 0; i < array.Length; i++)
                {
                    text += array[i];
                }
                return text;
            }

            private readonly ComputeShipDesignComposition computeShipDesignComposition;

            private readonly string[] engineSupport = new string[]
			{
				"Engine",
				"IncreaseSpeed"
			};

            private readonly string[] invisibilitySupport = new string[]
			{
				"Invisibility",
				"FleetEngine",
				"FlotillaRepair",
				"FlotillaShield",
				"Bait",
				"Repair"
			};

            private readonly string[] detectionSupport = new string[]
			{
				"Detection",
				"FleetEngine",
				"FlotillaRepair",
				"FlotillaShield",
				"Bait",
				"Repair"
			};

            private readonly string[] energyAttackerSupport = new string[]
			{
				"FleetEnhancer",
				"Honor",
				"EnergyEnhancer",
				"Jammer",
				"Repair"
			};

            private readonly string[] projectileAttackerSupport = new string[]
			{
				"FleetEnhancer",
				"Honor",
				"ProjectileEnhancer",
				"Jammer",
				"Repair"
			};

            private readonly string[] defenserShieldSupport = new string[]
			{
				"FleetEngine",
				"QuestDefenseMatrix",
				"FlotillaShield",
				"ShieldReload",
				"FleetEngine",
				"FlotillaRepair",
				"Bait",
				"Repair"
			};

            private readonly string[] defenserHullSupport = new string[]
			{
				"QuestDefenseMatrix",
				"FleetEngine",
				"FlotillaRepair",
				"FlotillaShield",
				"Bait",
				"Repair"
			};

            private readonly string[] colonizerSupport = new string[]
			{
				"Colonization"
			};

            private readonly string[] explorerSupport = new string[]
			{
				"Exploration"
			};

            private readonly string[] invaderSupport = new string[]
			{
				"Crew"
			};

            private readonly string[] siegeSupport = new string[]
			{
				"SupportCrewSpecial",
				"Siege",
				"Crew"
			};

            private readonly string[] motherShipSupport = new string[]
			{
				"MothershipLifeforce",
				"MothershipIndustry",
				"MothershipDust",
				"MothershipScience",
				"MothershipInfluence"
			};

            private readonly string[] motherShipAnchored = new string[]
			{
				"MothershipAnchoredDefense",
				"MothershipLifeforce"
			};

            private readonly string[] motherShipIndustry = new string[]
			{
				"MothershipIndustry"
			};

            private readonly string[] motherShipDust = new string[]
			{
				"MothershipDust"
			};

            private readonly string[] motherShipScience = new string[]
			{
				"MothershipScience"
			};

            private readonly string[] leecherSupport = new string[]
			{
				"GrowthLogistics"
			};

            private readonly string[] rootSupport = new string[]
			{
				"RootCreation"
			};

            private readonly string[] destroyerAttack = new string[]
			{
				"Destruction"
			};

            private readonly string[] fighter = new string[]
			{
				"Fighter"
			};

            private readonly string[] bomber = new string[]
			{
				"Bomber"
			};

            private readonly string[] juggernautMining = new string[]
			{
				"MiningProbe"
			};

            private readonly string[] juggernautSalvage = new string[]
			{
				"Salvager"
			};

            private readonly string[] juggernautTerraformation = new string[]
			{
				"Rejuvenator",
				"Terraformation"
			};

            private readonly string[] juggernautResearch = new string[]
			{
				"JuggernautResearch"
			};

            private readonly string[] juggernautAnalyser = new string[]
			{
				"JuggernautAnalyzer"
			};

            private readonly string[] juggernautFIDS = new string[]
			{
				"SystemSupportFIDS"
			};

            private readonly string[] anomalyReduction = new string[]
			{
				"AnomalyReduction"
			};

            private readonly string[] tractorBeam = new string[]
			{
				"TractorBeam",
				"FleetEnhancer",
				"Honor",
				"Jammer",
				"Repair"
			};
        }
    }
}
