﻿using System;
using System.Collections.Generic;
using Amplitude.AI.Common.DataStructure;
using Amplitude.AI.Core.Snapshots.DatabaseSnapshots;

namespace Amplitude.AI.Core.Brain.Strategic.Jobs.GlobalMilitary
{
    public class ModuleDecisionHelper
    {
        public void setModuleSnapshot(ModuleSnapshot mSnapshot)
        {
            this.moduleSnapshot = mSnapshot;
        }

        public void UpdateModules()
        {
            if (this.category == ShipLayoutSlotSnapshot.ModuleCategory.Defense && this.moduleSnapshot.Family.StartsWith("Hull"))
            {
                this.lastModuleDefenseSlotted = "Hull";
            }
            if (this.category == ShipLayoutSlotSnapshot.ModuleCategory.Defense && this.moduleSnapshot.Family.StartsWith("Shield"))
            {
                this.lastModuleDefenseSlotted = "Shield";
            }
            if (this.moduleSnapshot.Name.Contains("Strategic") && !this.hasName(this.moduleSnapshot.Family, this.allowMoreStrategics))
            {
                this.moduleNamesUsed.Add(this.moduleSnapshot.Name);
                this.moduleFamiliesUsed.Add(this.moduleSnapshot.Family);
            }
        }

        public bool IsModuleAllowed()
        {
            if (this.category == ShipLayoutSlotSnapshot.ModuleCategory.Defense && this.lastModuleDefenseSlotted.Length > 0 && this.moduleSnapshot.Family.StartsWith(this.lastModuleDefenseSlotted))
            {
                return false;
            }
            if (this.moduleNamesUsed.Contains(this.moduleSnapshot.Name))
            {
                return false;
            }
            if (this.hasName(this.moduleSnapshot.Family, this.oneOnlyTypes))
            {
                foreach (string input in this.moduleFamiliesUsed)
                {
                    if (this.hasName(input, this.oneOnlyTypes))
                    {
                        return false;
                    }
                }
                return true;
            }
            return true;
        }

        private bool hasName(string input, string[] names)
        {
            for (int i = 0; i < names.Length; i++)
            {
                if (input.Contains(names[i]))
                {
                    return true;
                }
            }
            return false;
        }

        public ModuleDecisionHelper()
        {
            this.lastModuleDefenseSlotted = "";
            this.moduleNamesUsed = new List<string>();
            this.moduleFamiliesUsed = new List<string>();
            this.category = ShipLayoutSlotSnapshot.ModuleCategory.None;
            this.moduleSnapshot = null;
        }

        public void setModuleCategory(ShipLayoutSlotSnapshot.ModuleCategory cat)
        {
            this.category = cat;
        }

        private string lastModuleDefenseSlotted;

        private List<string> moduleNamesUsed;

        private List<string> moduleFamiliesUsed;

        private ShipLayoutSlotSnapshot.ModuleCategory category;

        private ModuleSnapshot moduleSnapshot;

        private readonly string[] oneOnlyTypes = new string[]
		{
			"FleetEngine",
			"QuestDefenseMatrix",
			"Bait",
			"AntiCarrier",
			"Jammer",
			"SquadronOffenseEnhancer",
			"SquadronDefenseEnhancer",
			"EnergyEnhancer",
			"ProjectileEnhancer",
			"KineticRailgun",
			"MissileSwarm"
		};

        private readonly string[] allowMoreStrategics = new string[]
		{
			"Fighter",
			"Bomber",
			"Siege"
		};
    }
}
