﻿using System;
using Amplitude.AI.Core.AIPlayers.MajorEmpire;
using Amplitude.AI.Core.Brain.Analysis.Data;
using Amplitude.AI.Core.Brain.Generators.Science;
using Amplitude.AI.Core.Brain.Goals;
using Amplitude.AI.Core.Decision;
using Amplitude.AI.Core.Decision.ProcessingPasses;
using Amplitude.AI.Core.Framework;
using Amplitude.AI.Core.Framework.Debug;
using Amplitude.AI.Core.Modules;
using Amplitude.AI.Core.Services;
using Amplitude.AI.Core.Snapshots;
using Amplitude.AI.Core.Snapshots.DatabaseSnapshots;
using Amplitude.AI.Core.Snapshots.EntitySnapshots.Empire;

namespace Amplitude.AI.Core.Brain.Generators.Military
{
    public class QueueTechnologyForIncreaseModuleEfficiency : ScienceBasedGenerator<ModuleSnapshot, Goal, GenerationData<ModuleSnapshot, Goal, NeedTechnology>>
    {
        public override string LocalizedName
        {
            get
            {
                return "Queue technology in order to have better module";
            }
        }

        public override string Description
        {
            get
            {
                return "Queue technology in order to have better module";
            }
        }

        public override int EvaluationPriority
        {
            get
            {
                return 0;
            }
        }

        protected override void GetContexts(AIPlayer aiPlayer)
        {
            AIPlayer_MajorEmpire aiplayer_MajorEmpire = aiPlayer as AIPlayer_MajorEmpire;
            if (!(aiPlayer.ControlledEmpireSnapshot is MajorEmpireSnapshot) || aiplayer_MajorEmpire == null || !aiplayer_MajorEmpire.StrategicDataHolder.MilitaryData.HasTaskForcePattern)
            {
                return;
            }
            IDatabase<ModuleSnapshot> database;
            if (!Amplitude.AI.Core.Framework.Services.GetService<IDatabaseRepositoryService>().TryGetDatabase<ModuleSnapshot>(out database))
            {
                Diagnostics.LogError("Cannot retrieved 'Snapshots.DatabaseSnapshots.ModuleSnapshot' database.");
                return;
            }
            foreach (ModuleSnapshot moduleSnapshot in database)
            {
                bool flag = false;
                bool flag2 = false;
                for (int i = 0; i < aiPlayer.AnalysisDataHolder.MyEmpireData.AvailableModules.Count; i++)
                {
                    ModuleSnapshot moduleSnapshot2 = aiPlayer.AnalysisDataHolder.MyEmpireData.AvailableModules[i];
                    if (moduleSnapshot2.Family == moduleSnapshot.Family)
                    {
                        flag = true;
                        if (moduleSnapshot2.Level > moduleSnapshot.Level)
                        {
                            flag2 = true;
                        }
                    }
                }
                if (flag && !flag2)
                {
                    base.ContinueDecision(aiPlayer, moduleSnapshot);
                }
                bool flag3 = false;
                for (int j = 0; j < this.WantedModuleFamilly.Length; j++)
                {
                    if (this.WantedModuleFamilly[j] == moduleSnapshot.Family)
                    {
                        flag3 = true;
                        break;
                    }
                }
                int k = 0;
                while (k < this.WantedJuggernautModuleFamilly.Length)
                {
                    string text = this.WantedJuggernautModuleFamilly[k];
                    if (text == moduleSnapshot.Family)
                    {
                        if (this.WantJuggernautModule(aiPlayer, text))
                        {
                            flag3 = true;
                            break;
                        }
                        break;
                    }
                    else
                    {
                        k++;
                    }
                }
                if (!flag && flag3)
                {
                    base.ContinueDecision(aiPlayer, moduleSnapshot);
                }
            }
        }

        private bool WantJuggernautModule(AIPlayer aiPlayer, string wantedFamilly)
        {
            AIPlayer_MajorEmpire aiplayer_MajorEmpire = aiPlayer as AIPlayer_MajorEmpire;
            if (aiplayer_MajorEmpire.StrategicDataHolder.JuggernautsData.HaveJuggernaut(ShipPattern.ShipRole.SupportJuggernaut))
            {
                for (int i = 0; i < this.WantedSupportJuggernautModuleFamilly.Length; i++)
                {
                    if (this.WantedSupportJuggernautModuleFamilly[i] == wantedFamilly)
                    {
                        return true;
                    }
                }
            }
            if (aiplayer_MajorEmpire.StrategicDataHolder.JuggernautsData.HaveJuggernaut(ShipPattern.ShipRole.ScienceJuggernaut))
            {
                for (int j = 0; j < this.WantedScienceJuggernautModuleFamilly.Length; j++)
                {
                    if (this.WantedScienceJuggernautModuleFamilly[j] == wantedFamilly)
                    {
                        return true;
                    }
                }
            }
            if (aiplayer_MajorEmpire.StrategicDataHolder.JuggernautsData.HaveJuggernaut(ShipPattern.ShipRole.MilitaryJuggernaut))
            {
                for (int k = 0; k < this.WantedSupportJuggernautModuleFamilly.Length; k++)
                {
                    if (this.WantedSupportJuggernautModuleFamilly[k] == wantedFamilly)
                    {
                        return true;
                    }
                }
            }
            return false;
        }

        protected override void GetParents(AIPlayer aiPlayer, ModuleSnapshot context)
        {
            bool flag = false;
            for (int i = 0; i < this.WantedJuggernautModuleFamilly.Length; i++)
            {
                if (this.WantedJuggernautModuleFamilly[i] == context.Family)
                {
                    flag = true;
                    break;
                }
            }
            if (flag)
            {
                base.ContinueDecision(aiPlayer, context, aiPlayer.Blackboard.FindFirstRunning<UseJuggernaut>());
                return;
            }
            base.ContinueDecision(aiPlayer, context, aiPlayer.Blackboard.FindFirstRunning<ImproveCapabilities>());
        }

        protected override HeuristicFloat ComputeLocalMotivation(AIPlayer aiPlayer, GenerationData<ModuleSnapshot, Goal, NeedTechnology> generationData)
        {
            if (generationData.Parent is UseJuggernaut)
            {
                return new HeuristicFloat(1f, "");
            }
            if (generationData.Context is WeaponModuleSnapshot)
            {
                return new HeuristicFloat(0.7f, "");
            }
            if (generationData.Context is DefenseModuleSnapshot)
            {
                return new HeuristicFloat(0.6f, "");
            }
            if (generationData.Context is SupportModuleSnapshot && generationData.Context.Family.Contains("MothershipLifeforce"))
            {
                return new HeuristicFloat(1f, "");
            }
            if (generationData.Context is SupportModuleSnapshot && generationData.Context.Family.Contains("Mothership"))
            {
                return new HeuristicFloat(0.8f, "");
            }
            if (generationData.Context is SupportModuleSnapshot)
            {
                return new HeuristicFloat(0.5f, "");
            }
            if (generationData.Context is SquadronModuleSnapshot)
            {
                return new HeuristicFloat(0.6f, "");
            }
            return new HeuristicFloat(0.4f, "");
        }

        protected override bool IsValidConstructible(AIPlayer aiPlayer, ScienceImprovementData scienceImprovementData, ModuleSnapshot context, Goal parent)
        {
            string[] array;
            return scienceImprovementData.ConstructibleSnapshot.GainsSnapshot.HasGain("ModuleGain", context.Family) && scienceImprovementData.ConstructibleSnapshot.GainsSnapshot.TryGetGainTags("ModuleGain", out array) && int.Parse(array[0]) == context.Level && (array.Length != 3 || aiPlayer.ControlledEmpireSnapshot.GetResourceNet(array[2]) > 0f);
        }

        public QueueTechnologyForIncreaseModuleEfficiency()
        {
        }

        public readonly string[] WantedModuleFamilly = new string[]
		{
			"Engine",
			"EnergyEnhancer",
			"ProjectileEnhancer",
			"Jammer",
			"Repair",
			"ShieldReload",
			"FleetEngine",
			"FlotillaRepair",
			"FlotillaShield",
			"Bait",
			"Crew",
			"Siege",
			"Destruction",
			"Fighter",
			"Bomber",
			"DestructionChargeRate",
			"TractorBeam"
		};

        public readonly string[] WantedJuggernautModuleFamilly = new string[]
		{
			"MiningProbe",
			"Terraformation",
			"Salvager",
			"JuggernautResearch",
			"JuggernautAnalyzer",
			"SystemSupportFIDS",
			"Rejuvenator",
			"AnomalyReduction",
			"TractorBeam",
			"FleetEnhancer",
			"DestructionChargeRate",
			"TradeEnhancer",
			"Teleporter"
		};

        public readonly string[] WantedSupportJuggernautModuleFamilly = new string[]
		{
			"MiningProbe",
			"SystemSupportFIDS"
		};

        public readonly string[] WantedScienceJuggernautModuleFamilly = new string[]
		{
			"Terraformation",
			"Salvager",
			"JuggernautResearch",
			"JuggernautAnalyzer",
			"Rejuvenator",
			"AnomalyReduction"
		};

        public readonly string[] WantedMilitaryJuggernautModuleFamilly = new string[]
		{
			"TractorBeam"
		};
    }
}
