﻿using System;
using Amplitude.AI.Core.Brain.Allocators;
using Amplitude.AI.Core.Brain.Analysis.Data;
using Amplitude.AI.Core.Brain.Goals;
using Amplitude.AI.Core.Brain.Tasks.FleetMissionTasks;
using Amplitude.AI.Core.Decision;
using Amplitude.AI.Core.Decision.ProcessingPasses;
using Amplitude.AI.Core.Framework;
using Amplitude.AI.Core.Framework.Debug;
using Amplitude.AI.Core.Services;
using Amplitude.AI.Core.Snapshots;
using Amplitude.AI.Core.Snapshots.EntitySnapshots;
using Amplitude.AI.Core.Snapshots.EntitySnapshots.Colony;
using Amplitude.AI.Core.Snapshots.EntitySnapshots.Empire;
using Amplitude.AI.Core.Snapshots.EntitySnapshots.Galaxy;
using Amplitude.AI.Core.Snapshots.Position;

namespace Amplitude.AI.Core.Brain.Generators.Exploration
{
    public class SearchAllColonies : DefaultContextBasedGenerator<NodePosition, Win, SearchNodeCuriosities>
    {
        public override string LocalizedName
        {
            get
            {
                return "Search all the curiosities on owned systems.";
            }
        }

        public override string Description
        {
            get
            {
                return "Search all the curiosities on owned systems.";
            }
        }

        public override int EvaluationPriority
        {
            get
            {
                return 5;
            }
        }

        protected override void GetContexts(AIPlayer aiPlayer)
        {
            MajorEmpireSnapshot majorEmpireSnapshot = aiPlayer.ControlledEmpireSnapshot as MajorEmpireSnapshot;
            if (majorEmpireSnapshot == null)
            {
                return;
            }
            ISnapshotRepositoryService service = Amplitude.AI.Core.Framework.Services.GetService<ISnapshotRepositoryService>();
            this.indexToCuriosities = new int[majorEmpireSnapshot.ColonizedStarSystems.Count, 2];
            for (int i = 0; i < majorEmpireSnapshot.ColonizedStarSystems.Count; i++)
            {
                ColonizedStarSystemSnapshot colonizedStarSystemSnapshot = majorEmpireSnapshot.ColonizedStarSystems[i];
                GameNodeSnapshot gameNodeSnapshot;
                if (service.GalaxySnapshot.TryGetGameNode<GameNodeSnapshot>(colonizedStarSystemSnapshot.NodePosition, out gameNodeSnapshot) && colonizedStarSystemSnapshot.IsActive && gameNodeSnapshot.GetAnalysisData<NodeData>(aiPlayer.ControlledEmpireIndex).CuriosityToSearch.Count != 0)
                {
                    base.ContinueDecision(aiPlayer, colonizedStarSystemSnapshot.NodePosition);
                }
            }
            for (int j = 0; j < majorEmpireSnapshot.ColonizedStarSystems.Count; j++)
            {
                ColonizedStarSystemSnapshot colonizedStarSystemSnapshot2 = majorEmpireSnapshot.ColonizedStarSystems[j];
                this.indexToCuriosities[j, 0] = colonizedStarSystemSnapshot2.NodePosition.NodeIndex;
                GameNodeSnapshot gameNodeSnapshot2;
                if (service.GalaxySnapshot.TryGetGameNode<GameNodeSnapshot>(colonizedStarSystemSnapshot2.NodePosition, out gameNodeSnapshot2) && colonizedStarSystemSnapshot2.IsActive && gameNodeSnapshot2.GetAnalysisData<NodeData>(aiPlayer.ControlledEmpireIndex).CuriosityToSearch.Count != 0)
                {
                    this.indexToCuriosities[j, 1] = gameNodeSnapshot2.GetAnalysisData<NodeData>(aiPlayer.ControlledEmpireIndex).CuriosityToSearch.Count;
                }
                else
                {
                    this.indexToCuriosities[j, 1] = 0;
                }
            }
            for (int k = 0; k < majorEmpireSnapshot.ColonizedStarSystems.Count; k++)
            {
                ColonizedStarSystemSnapshot colonizedStarSystemSnapshot3 = majorEmpireSnapshot.ColonizedStarSystems[k];
                GameNodeSnapshot gameNodeSnapshot3;
                if (service.GalaxySnapshot.TryGetGameNode<GameNodeSnapshot>(colonizedStarSystemSnapshot3.NodePosition, out gameNodeSnapshot3) && this.indexToCuriosities[k, 1] > 0)
                {
                    base.ContinueDecision(aiPlayer, colonizedStarSystemSnapshot3.NodePosition);
                }
            }
        }

        protected override void GetParents(AIPlayer aiPlayer, NodePosition context)
        {
            base.ContinueDecision(aiPlayer, context, aiPlayer.Blackboard.FindFirstRunning<Win>(VictorySnapshot.VictoryType.Score));
        }

        protected override void GenerateDecision(AIPlayer aiPlayer, NodePosition context, Win parent)
        {
            SearchNodeCuriosities searchNodeCuriosities = aiPlayer.Blackboard.FindFirstRunning<SearchNodeCuriosities>(context);
            if (searchNodeCuriosities == null)
            {
                searchNodeCuriosities = new SearchNodeCuriosities(aiPlayer.Blackboard.GenerateGUID(), context);
                TaskWithInstance taskWithInstance = searchNodeCuriosities;
                ResourceAllocation[] array = new InstanceAllocation_Fleet[1];
                ResourceAllocation[] instanceAllocations = array;
                taskWithInstance.InstanceAllocations = instanceAllocations;
                searchNodeCuriosities.InstanceAllocations[0] = new InstanceAllocation_Fleet(ShipPattern.ShipRole.Exploration, 1);
                aiPlayer.Blackboard.Register<SearchNodeCuriosities>(searchNodeCuriosities);
            }
            base.ContinueDecision(aiPlayer, base.GetGenerationData(context, parent, searchNodeCuriosities));
        }

        protected override HeuristicFloat ComputeLocalMotivation(AIPlayer aiPlayer, GenerationData<NodePosition, Win, SearchNodeCuriosities> generationData)
        {
            int num = 0;
            int num2 = 0;
            for (int i = 0; i < this.indexToCuriosities.GetLength(0); i++)
            {
                num += this.indexToCuriosities[i, 1];
                if (this.indexToCuriosities[i, 0] == generationData.Context.NodeIndex)
                {
                    num2 += this.indexToCuriosities[i, 1];
                }
            }
            HeuristicFloat result = new HeuristicFloat(0.2f, "Base Interest");
            if (Amplitude.AI.Core.Framework.Services.GetService<ITimeService>().CurrentTurn > Duration.FromNormalSpeed(10).Value)
            {
                if (num2 > 1)
                {
                    result.Boost(0.04f * (float)(num2 - 1), "Extra Curiosities in the same system");
                }
                if (num > 1)
                {
                    result.Boost(0.01f * (float)(num - 1), "Extra Curiosities in the empire");
                    result.Clamp01();
                }
            }
            return result;
        }

        public SearchAllColonies()
        {
        }

        private int[,] indexToCuriosities;
    }
}
